from __future__ import absolute_import, unicode_literals

from copy import deepcopy

import responses
from transifex.api.jsonapi import JsonApi, Resource
from transifex.api.jsonapi.compat import abc

from .constants import host
from .payloads import Payloads


class ATestApi(JsonApi):
    HOST = host


@ATestApi.register
class Child(Resource):
    TYPE = "children"


@ATestApi.register
class Parent(Resource):
    TYPE = "parents"


test_api = ATestApi(host=host, auth="test_api_key")


child_payloads = Payloads('children', singular_type="child")


PAYLOAD = {'data': {
    'type': "parents",
    'id': "1",
    'attributes': {'name': "parent 1"},
    'relationships': {'children': {
        'data': [{'type': "children", 'id': "1"},
                 {'type': "children", 'id': "2"}],
        'links': {'related': "/parents/1/children"},
    }},
}}


def make_simple_assertions(parent):
    assert isinstance(parent.children, abc.Sequence)
    assert parent.children[0].id == "1"
    assert parent.children[1].id == "2"


def test_plural_list():
    parent = test_api.Parent(PAYLOAD)
    make_simple_assertions(parent)
    parent = test_api.Parent(PAYLOAD['data'])
    make_simple_assertions(parent)
    parent = test_api.Parent(**PAYLOAD['data'])
    make_simple_assertions(parent)


def test_included():
    payload = deepcopy(PAYLOAD)
    payload['included'] = child_payloads[1:3]
    parent = test_api.Parent(payload)
    make_simple_assertions(parent)

    assert parent.children[0].name == "child 1"
    assert parent.children[1].name == "child 2"


@responses.activate
def test_refetch():
    responses.add(responses.GET, "{}/parents/1/children".format(host),
                  json={'data': child_payloads[1:4]})

    parent = test_api.Parent(PAYLOAD)
    assert len(parent.children) == 2

    parent.fetch('children')
    assert len(parent.children) == 3
    assert ([child.name for child in parent.children] ==
            ["child {}".format(i) for i in range(1, 4)])


@responses.activate
def test_save_with_included():
    payload = deepcopy(PAYLOAD)
    payload['included'] = child_payloads[1:3]
    responses.add(responses.PATCH, "{}/parents/1".format(host), json=payload)
    parent = test_api.Parent(payload)
    parent.save(name="parent 1")
    assert [child.name for child in parent.children] == ["child 1", "child 2"]



from transifex.native.django.utils import lazy_translate as lazyt
from transifex.native.django.utils import translate as t
from transifex.native.django.utils import utranslate as ut

default_app_config = 'transifex.native.django.apps.NativeConfig'



from setuptools import find_packages, setup

import versioneer

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="transifex-python",
    author="Transifex",
    author_email="info@transifex.com",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description=("Transifex Python Toolkit"),
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    url="https://github.com/transifex/transifex-python",
    install_requires=["pyseeyou", "pytz", "requests", "click", "asttokens"],
)



from __future__ import absolute_import, unicode_literals

import json

import responses
from transifex.api.jsonapi import JsonApi, Resource

from .constants import host
from .payloads import Payloads


class ATestApi(JsonApi):
    HOST = host


@ATestApi.register
class BulkItem(Resource):
    TYPE = "bulk_items"


test_api = ATestApi(host=host, auth="test_api_key")


payloads = Payloads('bulk_items')


@responses.activate
def test_bulk_delete():
    responses.add(responses.DELETE, "{}/bulk_items".format(host))

    items = [test_api.BulkItem(payload) for payload in payloads[1:6]]
    test_api.BulkItem.bulk_delete([items[0],
                                   items[1].as_resource_identifier(),
                                   items[2].as_relationship(),
                                   items[3].id])

    assert len(responses.calls) == 1
    call = responses.calls[0]
    assert (call.request.headers['Content-Type'] ==
            'application/vnd.api+json;profile="bulk"')
    assert (json.loads(call.request.body.decode())['data'] ==
            [{'type': "bulk_items", 'id': str(i)} for i in range(1, 5)])


@responses.activate
def test_bulk_create():
    response_payload = payloads[1:5]
    for item, when in zip(response_payload, range(1, 5)):
        item['attributes']['created'] = "now + {}".format(when)
    responses.add(responses.POST, "{}/bulk_items".format(host),
                  json={'data': response_payload})

    result = test_api.BulkItem.bulk_create([
        BulkItem(attributes={'name': "bulk_item 1"}),
        {'attributes': {'name': "bulk_item 2"}},
        ({'name': "bulk_item 3"}, None),
        {'name': "bulk_item 4"},
    ])

    assert len(responses.calls) == 1
    call = responses.calls[0]
    assert (call.request.headers['Content-Type'] ==
            'application/vnd.api+json;profile="bulk"')
    assert (json.loads(call.request.body.decode()) ==
            {'data': [{'type': "bulk_items",
                       'attributes': {'name': "bulk_item {}".format(i)}}
                      for i in range(1, 5)]})

    assert isinstance(result[0], BulkItem)
    assert result[1].id == "2"
    assert result[2] == test_api.BulkItem(id="3")

    for i in range(4):
        assert result[i].id == str(i + 1)
        assert (result[i].name ==
                result[i].attributes['name'] ==
                "bulk_item {}".format(i + 1))
        assert (result[i].created ==
                result[i].attributes['created'] ==
                "now + {}".format(i + 1))


@responses.activate
def test_bulk_update():
    response_payload = payloads[1:6]
    for item, when in zip(response_payload, range(1, 6)):
        item['attributes'].update({'last_update': "now + {}".format(when),
                                   'name': "modified name {}".format(when)})
    responses.add(responses.PATCH, "{}/bulk_items".format(host),
                  json={'data': response_payload})

    result = test_api.BulkItem.bulk_update([
        test_api.BulkItem(id="1", attributes={'name': "modified name 1"}),
        {'id': "2", 'attributes': {'name': "modified name 2"}},
        ("3", {'name': "modified name 3"}, None),
        ("4", {'name': "modified name 4"}),
        "5",
    ])

    assert len(responses.calls) == 1
    call = responses.calls[0]
    assert (call.request.headers['Content-Type'] ==
            'application/vnd.api+json;profile="bulk"')
    assert (json.loads(call.request.body.decode()) ==
            {'data': ([{'type': "bulk_items",
                        'id': str(i),
                        'attributes': {'name': "modified name {}".format(i)}}
                       for i in range(1, 5)] +
                      [{'type': "bulk_items", 'id': "5"}])})

    assert isinstance(result[0], BulkItem)
    assert result[1].id == "2"
    assert result[2] == test_api.BulkItem(id="3")
    assert result[3].name == "modified name 4"

    for i in range(5):
        assert result[i].id == str(i + 1)
        assert (result[i].name ==
                result[i].attributes['name'] ==
                "modified name {}".format(i + 1))
        assert (result[i].last_update ==
                result[i].attributes['last_update'] ==
                "now + {}".format(i + 1))



from __future__ import absolute_import, unicode_literals

import six

from .compat import abc


def is_resource(value):
    from .resources import Resource

    return isinstance(value, Resource)


def is_collection(value):
    from .collections import Collection

    return isinstance(value, Collection)


def is_dict(value):
    return isinstance(value, abc.Mapping)


def is_list(value):
    return isinstance(value, abc.Sequence) and not isinstance(value, six.string_types)


def is_null(value):
    return value is None


def has_data(value):
    return is_dict(value) and "data" in value


def has_links(value):
    return is_dict(value) and "links" in value


def is_resource_identifier(value):
    return is_dict(value) and {"type", "id"} <= set(value.keys())


def is_relationship(value):
    return (
        is_dict(value)
        and has_data("value")
        and is_dict(value["data"])
        and is_resource_identifier(value["data"])
    )


def is_related(value):
    """Determines if value can be considered a relationship in any way."""

    return is_resource(value) or is_resource_identifier(value) or is_relationship(value)


def is_related_list(value):
    if has_data(value):
        value = value["data"]

    return is_list(value) and all((is_related(item) for item in value))


def is_fetched(value):
    return is_resource(value) and (value.attributes or value.relationships)



from __future__ import absolute_import, unicode_literals

from copy import deepcopy


class Payloads(object):
    """ Usage:

        >>> payloads = Payloads('items')
        >>> payloads[1]
        <<< {'type': "items", 'id': "1", 'attributes': {'name': "item 1"}}
        >>> payloads[1:4]
        <<< [{'type': "items", 'id': "1", 'attributes': {'name': "item 1"}},
        ...  {'type': "items", 'id': "2", 'attributes': {'name': "item 2"}},
        ...  {'type': "items", 'id': "3", 'attributes': {'name': "item 3"}}]
    """

    def __init__(self, plural_type, singular_type=None, extra=None):
        if singular_type is None:
            singular_type = plural_type[:-1]  # items => item
        if extra is None:
            extra = {}

        self.plural_type = plural_type
        self.singular_type = singular_type
        self.extra = deepcopy(extra)

    def __getitem__(self, index):
        if isinstance(index, slice):
            start = index.start if index.start is not None else 1
            stop = index.stop
            step = index.step if index.step is not None else 1
            return [self._payload(i) for i in range(start, stop, step)]
        else:
            return self._payload(index)

    def _payload(self, i):
        result = {'type': self.plural_type,
                  'id': str(i),
                  'attributes': {'name': ("{} {}".
                                          format(self.singular_type, i))}}
        result.update(self.extra)
        return result
